(ns sample.core
    (:require [uikit.core :as uikit]
              [com.stuartsierra.component :as component]
              [com.github.harlanji.clojure_stack.greetings :as c])
  (:import ;[com.google.j2objc.net IosHttpURLConnection]
           [java.net URL]))
(def white (-> ($ UIColor)
               ($ :whiteColor)))

(declare abc say-hi)

(defn on-tap [scope]
      (println "Tap")
      (say-hi))

(defn- my-screen [g] [:UIView :main
                {:setBackgroundColor white
                 :constraints        ["C:button.centerx=main.centerx"
                                      "C:button.centery=main.centery"]}
                [(uikit/button 1) :button {:setTitle:forState [g 0]
                                           :gestures          {:UITapGestureRecognizer #'on-tap}}]])

(defn say-hi []
  (let [c (.openConnection (URL. "http://localhost:3000/hi"))]
    (println "hi: :)" (slurp (.getContent c)))))

(comment defn say-hi []
  (let [uri (str "http://localhost:3000/hi")
        session ($ ($ NSURLSession) :sharedSession)
        _ (println "session:" session)
        url ($ ($ NSURL) :URLWithString uri)
        _ (println "url:" url)
        req ($ session
               :dataTaskWithURL url
               :completionHandler (fn [a b c]
                                      (println "got hi :)")))
        _ (println "req:" req)
        done ($ req :resume)]

       (println "uh, done:" done)))

; https://github.com/google/j2objc/blob/master/jre_emul/Classes/com/google/j2objc/net/IosHttpURLConnection.java#L369
(comment "
      NSURLSession *session = [NSURLSession sharedSession];
      [[session dataTaskWithURL:[NSURL URLWithString:londonWeatherUrl]
        completionHandler:^(NSData *data,
                                   NSURLResponse *response,
                                   NSError *error) {
                                                    // handle response

                                                    }] resume];

")

(defrecord MainUI []
           component/Lifecycle
           (start [component]
                  (let [window (-> ($ UIWindow)
                                   ($ :alloc)
                                   ($ :initWithFrame (-> ($ UIScreen)
                                                         ($ :mainScreen)
                                                         ($ :bounds))))
                        nav (-> ($ UINavigationController)
                                ($ :new))

                        ]
                       ;(println greeting)
                       (doto window
                             ($ :makeKeyAndVisible)
                             ($ :setRootViewController nav)
                             ($ :setBackgroundColor white))
                       (reset! uikit/current-top-controller nav)
                       ($ nav :setNavigationBarHidden true)
                       (assoc component
                              ; FIXME replacing the greeting call with a string works.
                              :ui (uikit/nav-push (uikit/controller "Screen title" (my-screen "Hey hey!"))))))
           (stop [component]
                 ))


(defonce system (atom (component/system-map :ui (->MainUI))))

(defn main []
  (println "Hai! hello:" (c/hello-again))

  (let [{:keys [ui]} (swap! system component/start)]
    ui))